@@ -1,4 +1,4 @@
-#include "fast_library.h"
+#include "fast_library.hpp"
 #include <arpa/inet.h>
 #include <fstream>
 #include <iostream>
@@ -13,7 +13,10 @@
 #include <sys/types.h>
 #include <unistd.h>
 
-#include "all_logcpp_libraries.h"
+// For uname function
+#include <sys/utsname.h>
+
+#include "all_logcpp_libraries.hpp"
 
 #include <boost/asio.hpp>
 
@@ -308,11 +311,6 @@ std::string print_tcp_flags(uint8_t flag_value) {
         return "-";
     }
 
-    // cod from pfring.h
-    // (tcp->fin * TH_FIN_MULTIPLIER) + (tcp->syn * TH_SYN_MULTIPLIER) +
-    // (tcp->rst * TH_RST_MULTIPLIER) + (tcp->psh * TH_PUSH_MULTIPLIER) +
-    // (tcp->ack * TH_ACK_MULTIPLIER) + (tcp->urg * TH_URG_MULTIPLIER);
-
     /*
         // Required for decoding tcp flags
         #define TH_FIN_MULTIPLIER   0x01
@@ -503,14 +501,18 @@ std::string get_protocol_name_by_number(unsigned int proto_number) {
     return proto_name;
 }
 
-// exec command in shell
-std::vector<std::string> exec(std::string cmd) {
-    std::vector<std::string> output_list;
-
+// Exec command in shell and capture output
+bool exec(const std::string& cmd, std::vector<std::string>& output_list,  std::string& error_text) {
     FILE* pipe = popen(cmd.c_str(), "r");
-    if (!pipe) return output_list;
+
+    if (!pipe) {
+        // We need more details in case of failure
+        error_text = "error code: " + std::to_string(errno) + " error text: " + strerror(errno);
+        return false;
+    }
 
     char buffer[256];
+
     while (!feof(pipe)) {
         if (fgets(buffer, 256, pipe) != NULL) {
             size_t newbuflen = strlen(buffer);
@@ -525,7 +527,7 @@ std::vector<std::string> exec(std::string cmd) {
     }
 
     pclose(pipe);
-    return output_list;
+    return true;
 }
 
 bool print_pid_to_file(pid_t pid, std::string pid_path) {
@@ -614,7 +616,14 @@ interfaces_list_t get_interfaces_list() {
     // Format: 1: eth0: < ....
     boost::regex interface_name_pattern("^\\d+:\\s+(\\w+):.*?$");
 
-    std::vector<std::string> output_list = exec("ip -o link show");
+    std::string error_text;
+    std::vector<std::string> output_list;
+
+    bool exec_result = exec("ip -o link show", output_list, error_text);
+
+    if (!exec_result) {
+        return interfaces_list;
+    }
 
     if (output_list.empty()) {
         return interfaces_list;
@@ -636,7 +645,14 @@ interfaces_list_t get_interfaces_list() {
 ip_addresses_list_t get_ip_list_for_interface(std::string interface) {
     ip_addresses_list_t ip_list;
 
-    std::vector<std::string> output_list = exec("ip address show dev " + interface);
+    std::string error_text;
+    std::vector<std::string> output_list;
+
+    bool exec_result = exec("ip address show dev " + interface, output_list, error_text);
+
+    if (!exec_result) {
+        return ip_list;
+    }
 
     if (output_list.empty()) {
         return ip_list;
@@ -930,72 +946,6 @@ bool manage_interface_promisc_mode(std::string interface_name, bool switch_on) {
 
 #endif
 
-// Adds traffic speed to JSON structure
-void serialize_traffic_counters_to_json(json_object* jobj, const attack_details_t& current_attack) {
-    json_object_object_add(jobj, "total_incoming_traffic", json_object_new_int(current_attack.in_bytes));
-    json_object_object_add(jobj, "total_outgoing_traffic", json_object_new_int(current_attack.out_bytes));
-    json_object_object_add(jobj, "total_incoming_pps", json_object_new_int(current_attack.in_packets));
-    json_object_object_add(jobj, "total_outgoing_pps", json_object_new_int(current_attack.out_packets));
-    json_object_object_add(jobj, "total_incoming_flows", json_object_new_int(current_attack.in_flows));
-    json_object_object_add(jobj, "total_outgoing_flows", json_object_new_int(current_attack.out_flows));
-
-    json_object_object_add(jobj, "incoming_ip_fragmented_traffic", json_object_new_int(current_attack.fragmented_in_bytes));
-    json_object_object_add(jobj, "outgoing_ip_fragmented_traffic", json_object_new_int(current_attack.fragmented_out_bytes));
-    json_object_object_add(jobj, "incoming_ip_fragmented_pps", json_object_new_int(current_attack.fragmented_in_packets));
-    json_object_object_add(jobj, "outgoing_ip_fragmented_pps", json_object_new_int(current_attack.fragmented_out_packets));
-
-    json_object_object_add(jobj, "incoming_tcp_traffic", json_object_new_int(current_attack.tcp_in_bytes));
-    json_object_object_add(jobj, "outgoing_tcp_traffic", json_object_new_int(current_attack.tcp_out_bytes));
-    json_object_object_add(jobj, "incoming_tcp_pps", json_object_new_int(current_attack.tcp_in_packets));
-    json_object_object_add(jobj, "outgoing_tcp_pps", json_object_new_int(current_attack.tcp_out_packets));
-
-    json_object_object_add(jobj, "incoming_syn_tcp_traffic", json_object_new_int(current_attack.tcp_syn_in_bytes));
-    json_object_object_add(jobj, "outgoing_syn_tcp_traffic", json_object_new_int(current_attack.tcp_syn_out_bytes));
-    json_object_object_add(jobj, "incoming_syn_tcp_pps", json_object_new_int(current_attack.tcp_syn_in_packets));
-    json_object_object_add(jobj, "outgoing_syn_tcp_pps", json_object_new_int(current_attack.tcp_syn_out_packets));
-
-    json_object_object_add(jobj, "incoming_udp_traffic", json_object_new_int(current_attack.udp_in_bytes));
-    json_object_object_add(jobj, "outgoing_udp_traffic", json_object_new_int(current_attack.udp_out_bytes));
-    json_object_object_add(jobj, "incoming_udp_pps", json_object_new_int(current_attack.udp_in_packets));
-    json_object_object_add(jobj, "outgoing_udp_pps", json_object_new_int(current_attack.udp_out_packets));
-
-    json_object_object_add(jobj, "incoming_icmp_traffic", json_object_new_int(current_attack.icmp_in_bytes));
-    json_object_object_add(jobj, "outgoing_icmp_traffic", json_object_new_int(current_attack.icmp_out_bytes));
-    json_object_object_add(jobj, "incoming_icmp_pps", json_object_new_int(current_attack.icmp_in_packets));
-    json_object_object_add(jobj, "outgoing_icmp_pps", json_object_new_int(current_attack.icmp_out_packets));
-}
-
-json_object* serialize_attack_description_to_json(attack_details_t& current_attack) {
-    json_object* jobj = json_object_new_object();
-
-    attack_type_t attack_type         = detect_attack_type(current_attack);
-    std::string printable_attack_type = get_printable_attack_name(attack_type);
-
-    json_object_object_add(jobj, "attack_type", json_object_new_string(printable_attack_type.c_str()));
-    
-    json_object_object_add(jobj, "initial_attack_power", json_object_new_int(current_attack.attack_power));
-    
-    json_object_object_add(jobj, "peak_attack_power", json_object_new_int(current_attack.max_attack_power));
-    
-    json_object_object_add(jobj, "attack_direction",
-                           json_object_new_string(get_direction_name(current_attack.attack_direction).c_str()));
-    
-    json_object_object_add(jobj, "attack_protocol",
-                           json_object_new_string(get_printable_protocol_name(current_attack.attack_protocol).c_str()));
-
-    // Populate traffic counters
-    serialize_traffic_counters_to_json(jobj, current_attack);
-
-    json_object_object_add(jobj, "average_incoming_traffic", json_object_new_int(current_attack.average_in_bytes));
-    json_object_object_add(jobj, "average_outgoing_traffic", json_object_new_int(current_attack.average_out_bytes));
-    json_object_object_add(jobj, "average_incoming_pps", json_object_new_int(current_attack.average_in_packets));
-    json_object_object_add(jobj, "average_outgoing_pps", json_object_new_int(current_attack.average_out_packets));
-    json_object_object_add(jobj, "average_incoming_flows", json_object_new_int(current_attack.average_in_flows));
-    json_object_object_add(jobj, "average_outgoing_flows", json_object_new_int(current_attack.average_out_flows));
-
-    return jobj;
-}
-
 std::string serialize_attack_description(attack_details_t& current_attack) {
     std::stringstream attack_description;
 
@@ -1008,10 +958,10 @@ std::string serialize_attack_description(attack_details_t& current_attack) {
                        << "Attack direction: " << get_direction_name(current_attack.attack_direction) << "\n"
                        << "Attack protocol: " << get_printable_protocol_name(current_attack.attack_protocol) << "\n";
 
-    attack_description << "Total incoming traffic: " << convert_speed_to_mbps(current_attack.in_bytes) << " mbps\n"
-                       << "Total outgoing traffic: " << convert_speed_to_mbps(current_attack.out_bytes) << " mbps\n"
-                       << "Total incoming pps: " << current_attack.in_packets << " packets per second\n"
-                       << "Total outgoing pps: " << current_attack.out_packets << " packets per second\n"
+    attack_description << "Total incoming traffic: " << convert_speed_to_mbps(current_attack.total.in_bytes) << " mbps\n"
+                       << "Total outgoing traffic: " << convert_speed_to_mbps(current_attack.total.out_bytes) << " mbps\n"
+                       << "Total incoming pps: " << current_attack.total.in_packets << " packets per second\n"
+                       << "Total outgoing pps: " << current_attack.total.out_packets << " packets per second\n"
                        << "Total incoming flows: " << current_attack.in_flows << " flows per second\n"
                        << "Total outgoing flows: " << current_attack.out_flows << " flows per second\n";
 
@@ -1025,29 +975,29 @@ std::string serialize_attack_description(attack_details_t& current_attack) {
                        << "Average outgoing flows: " << current_attack.average_out_flows << " flows per second\n";
 
     attack_description
-        << "Incoming ip fragmented traffic: " << convert_speed_to_mbps(current_attack.fragmented_in_bytes) << " mbps\n"
-        << "Outgoing ip fragmented traffic: " << convert_speed_to_mbps(current_attack.fragmented_out_bytes) << " mbps\n"
-        << "Incoming ip fragmented pps: " << current_attack.fragmented_in_packets << " packets per second\n"
-        << "Outgoing ip fragmented pps: " << current_attack.fragmented_out_packets << " packets per second\n"
+        << "Incoming ip fragmented traffic: " << convert_speed_to_mbps(current_attack.fragmented.in_bytes) << " mbps\n"
+        << "Outgoing ip fragmented traffic: " << convert_speed_to_mbps(current_attack.fragmented.out_bytes) << " mbps\n"
+        << "Incoming ip fragmented pps: " << current_attack.fragmented.in_packets << " packets per second\n"
+        << "Outgoing ip fragmented pps: " << current_attack.fragmented.out_packets << " packets per second\n"
 
-        << "Incoming tcp traffic: " << convert_speed_to_mbps(current_attack.tcp_in_bytes) << " mbps\n"
-        << "Outgoing tcp traffic: " << convert_speed_to_mbps(current_attack.tcp_out_bytes) << " mbps\n"
-        << "Incoming tcp pps: " << current_attack.tcp_in_packets << " packets per second\n"
-        << "Outgoing tcp pps: " << current_attack.tcp_out_packets << " packets per second\n"
-        << "Incoming syn tcp traffic: " << convert_speed_to_mbps(current_attack.tcp_syn_in_bytes) << " mbps\n"
-        << "Outgoing syn tcp traffic: " << convert_speed_to_mbps(current_attack.tcp_syn_out_bytes) << " mbps\n"
-        << "Incoming syn tcp pps: " << current_attack.tcp_syn_in_packets << " packets per second\n"
-        << "Outgoing syn tcp pps: " << current_attack.tcp_syn_out_packets << " packets per second\n"
+        << "Incoming tcp traffic: " << convert_speed_to_mbps(current_attack.tcp.in_bytes) << " mbps\n"
+        << "Outgoing tcp traffic: " << convert_speed_to_mbps(current_attack.tcp.out_bytes) << " mbps\n"
+        << "Incoming tcp pps: " << current_attack.tcp.in_packets << " packets per second\n"
+        << "Outgoing tcp pps: " << current_attack.tcp.out_packets << " packets per second\n"
+        << "Incoming syn tcp traffic: " << convert_speed_to_mbps(current_attack.tcp_syn.in_bytes) << " mbps\n"
+        << "Outgoing syn tcp traffic: " << convert_speed_to_mbps(current_attack.tcp_syn.out_bytes) << " mbps\n"
+        << "Incoming syn tcp pps: " << current_attack.tcp_syn.in_packets << " packets per second\n"
+        << "Outgoing syn tcp pps: " << current_attack.tcp_syn.out_packets << " packets per second\n"
 
-        << "Incoming udp traffic: " << convert_speed_to_mbps(current_attack.udp_in_bytes) << " mbps\n"
-        << "Outgoing udp traffic: " << convert_speed_to_mbps(current_attack.udp_out_bytes) << " mbps\n"
-        << "Incoming udp pps: " << current_attack.udp_in_packets << " packets per second\n"
-        << "Outgoing udp pps: " << current_attack.udp_out_packets << " packets per second\n"
+        << "Incoming udp traffic: " << convert_speed_to_mbps(current_attack.udp.in_bytes) << " mbps\n"
+        << "Outgoing udp traffic: " << convert_speed_to_mbps(current_attack.udp.out_bytes) << " mbps\n"
+        << "Incoming udp pps: " << current_attack.udp.in_packets << " packets per second\n"
+        << "Outgoing udp pps: " << current_attack.udp.out_packets << " packets per second\n"
 
-        << "Incoming icmp traffic: " << convert_speed_to_mbps(current_attack.icmp_in_bytes) << " mbps\n"
-        << "Outgoing icmp traffic: " << convert_speed_to_mbps(current_attack.icmp_out_bytes) << " mbps\n"
-        << "Incoming icmp pps: " << current_attack.icmp_in_packets << " packets per second\n"
-        << "Outgoing icmp pps: " << current_attack.icmp_out_packets << " packets per second\n";
+        << "Incoming icmp traffic: " << convert_speed_to_mbps(current_attack.icmp.in_bytes) << " mbps\n"
+        << "Outgoing icmp traffic: " << convert_speed_to_mbps(current_attack.icmp.out_bytes) << " mbps\n"
+        << "Incoming icmp pps: " << current_attack.icmp.in_packets << " packets per second\n"
+        << "Outgoing icmp pps: " << current_attack.icmp.out_packets << " packets per second\n";
 
     return attack_description.str();
 }
@@ -1056,23 +1006,23 @@ attack_type_t detect_attack_type(attack_details_t& current_attack) {
     double threshold_value = 0.9;
 
     if (current_attack.attack_direction == INCOMING) {
-        if (current_attack.tcp_syn_in_packets > threshold_value * current_attack.in_packets) {
+        if (current_attack.tcp_syn.in_packets > threshold_value * current_attack.total.in_packets) {
             return ATTACK_SYN_FLOOD;
-        } else if (current_attack.icmp_in_packets > threshold_value * current_attack.in_packets) {
+        } else if (current_attack.icmp.in_packets > threshold_value * current_attack.total.in_packets) {
             return ATTACK_ICMP_FLOOD;
-        } else if (current_attack.fragmented_in_packets > threshold_value * current_attack.in_packets) {
+        } else if (current_attack.fragmented.in_packets > threshold_value * current_attack.total.in_packets) {
             return ATTACK_IP_FRAGMENTATION_FLOOD;
-        } else if (current_attack.udp_in_packets > threshold_value * current_attack.in_packets) {
+        } else if (current_attack.udp.in_packets > threshold_value * current_attack.total.in_packets) {
             return ATTACK_UDP_FLOOD;
         }
     } else if (current_attack.attack_direction == OUTGOING) {
-        if (current_attack.tcp_syn_out_packets > threshold_value * current_attack.out_packets) {
+        if (current_attack.tcp_syn.out_packets > threshold_value * current_attack.total.out_packets) {
             return ATTACK_SYN_FLOOD;
-        } else if (current_attack.icmp_out_packets > threshold_value * current_attack.out_packets) {
+        } else if (current_attack.icmp.out_packets > threshold_value * current_attack.total.out_packets) {
             return ATTACK_ICMP_FLOOD;
-        } else if (current_attack.fragmented_out_packets > threshold_value * current_attack.out_packets) {
+        } else if (current_attack.fragmented.out_packets > threshold_value * current_attack.total.out_packets) {
             return ATTACK_IP_FRAGMENTATION_FLOOD;
-        } else if (current_attack.udp_out_packets > threshold_value * current_attack.out_packets) {
+        } else if (current_attack.udp.out_packets > threshold_value * current_attack.total.out_packets) {
             return ATTACK_UDP_FLOOD;
         }
     }
@@ -1096,7 +1046,7 @@ std::string get_printable_attack_name(attack_type_t attack) {
     }
 }
 
-std::string serialize_network_load_to_text(map_element_t& network_speed_meter, bool average) {
+std::string serialize_network_load_to_text(subnet_counter_t& network_speed_meter, bool average) {
     std::stringstream buffer;
 
     std::string prefix = "Network";
@@ -1105,25 +1055,14 @@ std::string serialize_network_load_to_text(map_element_t& network_speed_meter, b
         prefix = "Average network";
     }
 
-    buffer << prefix << " incoming traffic: " << convert_speed_to_mbps(network_speed_meter.in_bytes) << " mbps\n"
-           << prefix << " outgoing traffic: " << convert_speed_to_mbps(network_speed_meter.out_bytes) << " mbps\n"
-           << prefix << " incoming pps: " << network_speed_meter.in_packets << " packets per second\n"
-           << prefix << " outgoing pps: " << network_speed_meter.out_packets << " packets per second\n";
+    buffer << prefix << " incoming traffic: " << convert_speed_to_mbps(network_speed_meter.total.in_bytes) << " mbps\n"
+           << prefix << " outgoing traffic: " << convert_speed_to_mbps(network_speed_meter.total.out_bytes) << " mbps\n"
+           << prefix << " incoming pps: " << network_speed_meter.total.in_packets << " packets per second\n"
+           << prefix << " outgoing pps: " << network_speed_meter.total.out_packets << " packets per second\n";
 
     return buffer.str();
 }
 
-json_object* serialize_network_load_to_json(map_element_t& network_speed_meter) {
-    json_object* jobj = json_object_new_object();
-
-    json_object_object_add(jobj, "incoming traffic", json_object_new_int(network_speed_meter.in_bytes));
-    json_object_object_add(jobj, "outgoing traffic", json_object_new_int(network_speed_meter.out_bytes));
-    json_object_object_add(jobj, "incoming pps", json_object_new_int(network_speed_meter.in_packets));
-    json_object_object_add(jobj, "outgoing pps", json_object_new_int(network_speed_meter.out_packets));
-
-    return jobj;
-}
-
 std::string serialize_statistic_counters_about_attack(attack_details_t& current_attack) {
     std::stringstream attack_description;
 
@@ -1537,7 +1476,8 @@ bool execute_web_request_secure(std::string address,
                                 std::string post_data,
                                 uint32_t& response_code,
                                 std::string& response_body,
-                                std::map<std::string, std::string>& headers) {
+                                std::map<std::string, std::string>& headers,
+                                std::string& error_text) {
 
     extern log4cpp::Category& logger;
 
@@ -1578,13 +1518,11 @@ bool execute_web_request_secure(std::string address,
         boost::asio::io_context ioc;
 
         // The SSL context is required, and holds certificates
-        boost::asio::ssl::context ctx{ boost::asio::ssl::context::sslv23_client };
+        boost::asio::ssl::context ctx{ boost::asio::ssl::context::tls_client };
 
         // Load default CA certificates
         ctx.set_default_verify_paths();
 
-        boost::asio::ip::tcp::resolver r(ioc);
-
         boost::asio::ip::tcp::resolver resolver{ ioc };
         boost::asio::ssl::stream<boost::asio::ip::tcp::socket> stream{ ioc, ctx };
 
@@ -1595,14 +1533,14 @@ bool execute_web_request_secure(std::string address,
             return false;
         }
 
-        auto end_point = r.resolve(boost::asio::ip::tcp::resolver::query{ host, port }, ec);
+        auto end_point = resolver.resolve(boost::asio::ip::tcp::resolver::query{ host, port }, ec);
 
         if (ec) {
             logger << log4cpp::Priority::ERROR << "Could not resolve peer address in execute_web_request " << ec;
             return false;
         }
 
-        logger << log4cpp::Priority::INFO << "Resolved domain to " << end_point.size() << " IP addresses";
+        logger << log4cpp::Priority::DEBUG << "Resolved host " <<  host << " to " << end_point.size() << " IP addresses";
 
         boost::asio::connect(stream.next_layer(), end_point.begin(), end_point.end(), ec);
 
@@ -1641,7 +1579,16 @@ bool execute_web_request_secure(std::string address,
             req.body() = post_data;
         }
 
-        req.set(boost::beast::http::field::content_type, "application/x-www-form-urlencoded");
+        std::string content_type = "application/x-www-form-urlencoded";
+
+        // We can override Content Type from headers
+        auto header_itr = headers.find("Content-Type");
+        
+        if (header_itr != headers.end()) {
+            content_type = header_itr->second;
+        }
+
+        req.set(boost::beast::http::field::content_type, content_type);
 
         // We must specify port explicitly if we use non standard one
         std::string full_host = host + ":" + std::to_string(stream.next_layer().remote_endpoint().port());
@@ -1677,9 +1624,10 @@ bool execute_web_request_secure(std::string address,
         // Return response body to caller
         response_body = resp.body();
 
-        // logger << log4cpp::Priority::INFO << "Response code: " << response_code;
+        logger << log4cpp::Priority::DEBUG << "Response code: " << response_code;
+
+        logger << log4cpp::Priority::DEBUG << "Prepare to shutdown TLS";
 
-        // Gracefully close the stream
         stream.shutdown(ec);
         if (ec == boost::asio::error::eof) {
             // Rationale:
@@ -1687,15 +1635,15 @@ bool execute_web_request_secure(std::string address,
             ec.assign(0, ec.category());
         }
 
-        if (ec) {
-            logger << log4cpp::Priority::DEBUG << "Can't shutdown connection gracefully: " << ec.message();
-            // But we should not return error to caller in this case because we pushed data properly
-        }
+        logger << log4cpp::Priority::DEBUG << "Successfully closed TLS";
 
         return true;
     } catch (std::exception& e) {
         logger << log4cpp::Priority::ERROR << "execute_web_request failed with error: " << e.what();
         return false;
+    } catch (...) {
+        logger << log4cpp::Priority::ERROR << "execute_web_request failed with unknown error";
+        return false;
     }
 
     return false;
@@ -1714,7 +1662,7 @@ bool execute_web_request(std::string address,
     std::string port = "http";
 
     if (address.find("https://") != std::string::npos) {
-        return execute_web_request_secure(address, request_type, post_data, response_code, response_body, headers);
+        return execute_web_request_secure(address, request_type, post_data, response_code, response_body, headers, error_text);
     }
 
     if (address.find("http://") == std::string::npos) {
@@ -1788,7 +1736,17 @@ bool execute_web_request(std::string address,
             req.body() = post_data;
         }
 
-        req.set(boost::beast::http::field::content_type, "application/x-www-form-urlencoded");
+        std::string content_type = "application/x-www-form-urlencoded";
+    
+        // We can override Content Type from headers
+        auto header_itr = headers.find("Content-Type");
+
+        if (header_itr != headers.end()) {
+            content_type = header_itr->second;
+        }
+
+        req.set(boost::beast::http::field::content_type, content_type);
+
         req.set(boost::beast::http::field::host, host + ":" + std::to_string(sock.remote_endpoint().port()));
         req.set(boost::beast::http::field::user_agent, "FastNetMon");
 
@@ -1828,6 +1786,9 @@ bool execute_web_request(std::string address,
         error_text = "execute_web_request failed with error: ";
         error_text += e.what();
         return false;
+    } catch (...) {
+        error_text = "execute_web_request failed with unknown error";
+        return false;
     }
 
     return false;
@@ -1899,3 +1860,397 @@ std::string join_by_comma_and_equal(std::map<std::string, std::string>& data) {
 
     return buffer.str();
 }
+
+// We will store option name as key and value will be memory size in bytes
+bool parse_meminfo_into_map(std::map<std::string, uint64_t>& parsed_meminfo) {
+    extern log4cpp::Category& logger;
+
+    std::ifstream meminfo_file("/proc/meminfo");
+    boost::regex memory_info_pattern("^(.*?):\\s+(\\d+).*$", boost::regex::icase);
+
+    if (!meminfo_file.is_open()) {
+        logger << log4cpp::Priority::ERROR << "Could not open meminfo file";
+        return false;
+    }
+
+    std::string line;
+
+    while (getline(meminfo_file, line)) {
+        // MemTotal:         501912 kB
+        boost::match_results<std::string::const_iterator> regex_results;
+
+        if (boost::regex_match(line, regex_results, memory_info_pattern)) {
+            uint64_t memory_value = 0;
+
+            bool integer_parser_result = read_uint64_from_string(regex_results[2], memory_value);
+
+            if (!integer_parser_result) {
+                logger << log4cpp::Priority::ERROR << "Could not parse " << regex_results[2] << " as unsigned 64 bit integer";
+                return false;
+            }
+
+            parsed_meminfo[regex_results[1]] = memory_value * 1024;
+        }
+    }
+
+
+    return true;
+}
+
+// Reads uint64_t from string with all required safety checks
+bool read_uint64_from_string(const std::string& line, uint64_t& value) {
+    uint64_t temp_value = 0;
+
+    try {
+        // Read value to intermediate variable to avoid interference with argument of function in case of failure
+        // NB! This function does not work very well when we have minus in input sequence as it will accept it
+        // If the minus sign was part of the input sequence, the numeric value calculated from the sequence of digits
+        // is negated as if by unary minus in the result type, which applies unsigned integer wraparound rules.
+        temp_value = std::stoull(line);
+    } catch (...) {
+        return false;
+    }
+
+    value = temp_value;
+
+    return true;
+}
+
+bool read_file_to_string(const std::string& file_path, std::string& file_content) {
+    std::ifstream file_handler;
+
+    file_handler.open(file_path, std::ios::in);
+
+    if (file_handler.is_open()) {
+        std::stringstream str_stream;
+
+        str_stream << file_handler.rdbuf();
+        file_handler.close();
+
+        file_content = str_stream.str();
+        return true;
+    } else {
+        return false;
+    }
+}
+
+bool read_integer_from_file(const std::string& file_path, int& value) {
+    std::string file_content_in_string;
+
+    bool read_file_to_string_result = read_file_to_string(file_path, file_content_in_string);
+
+    if (!read_file_to_string_result) {
+        return false;
+    }
+
+    int scanned_value = 0;
+
+    bool read_integer_from_file = convert_string_to_any_integer_safe(file_content_in_string, scanned_value);
+
+    if (!read_integer_from_file) {
+        return false;
+    }
+
+    value = scanned_value;
+    return true;
+}
+
+// Safe way to convert string to any integer
+bool convert_string_to_any_integer_safe(const std::string& line, int& value) {
+    int temp_value = 0;
+
+    try {
+        temp_value = std::stoi(line);
+    } catch (...) {
+        // Could not parse number correctly
+        return false;
+    }
+
+    value = temp_value;
+
+    return true;
+}
+
+// This function is useful when we start it from thread and detach and so we are not interested in error text and we need to discard it
+void exec_no_error_check(const std::string& cmd) {
+    std::string error_text;
+    std::vector<std::string> output_list;
+
+    exec(cmd, output_list, error_text);
+    return;
+}
+
+unsigned int get_logical_cpus_number() {
+    extern log4cpp::Category& logger;
+
+    std::ifstream cpuinfo_file("/proc/cpuinfo");
+    boost::regex processor_pattern("^processor.*?$");
+
+    if (!cpuinfo_file.is_open()) {
+        logger << log4cpp::Priority::ERROR << "License: could not open cpuinfo";
+        return 0;
+    }
+
+    std::string line;
+    unsigned int logical_cpus_number = 0;
+
+    while (getline(cpuinfo_file, line)) {
+        boost::cmatch what;
+
+        if (regex_match(line.c_str(), what, processor_pattern)) {
+            logical_cpus_number++;
+        }
+    }
+
+    return logical_cpus_number;
+}
+
+// Get server's total memory in megabytes
+unsigned int get_total_memory() {
+    extern log4cpp::Category& logger;
+
+    std::ifstream meminfo_file("/proc/meminfo");
+    boost::regex memory_info_pattern("^(.*?):\\s+(\\d+).*$", boost::regex::icase);
+
+    if (!meminfo_file.is_open()) {
+        logger << log4cpp::Priority::ERROR << "License: could not open meminfo file";
+        return 0;
+    }
+
+    std::string line;
+
+    while (getline(meminfo_file, line)) {
+        // MemTotal:         501912 kB
+        boost::match_results<std::string::const_iterator> regex_results;
+
+        if (boost::regex_match(line, regex_results, memory_info_pattern)) {
+            if (regex_results[1] == "MemTotal") {
+                int memory_amount = 0;
+
+                bool conversion_result = convert_string_to_any_integer_safe(regex_results[2], memory_amount);
+
+                if (!conversion_result) {
+                    logger << log4cpp::Priority::ERROR << "Could not parse integer value";
+                    return 0;
+                }
+
+                return unsigned(memory_amount / 1024);
+            }
+        } else {
+            logger << log4cpp::Priority::ERROR << "Could not parse line in /proc/meminfo: " << line;
+            return 0;
+        }
+    }
+
+    return 0;
+}
+
+// Return code name of Linux distro:
+// ID=debian
+// ID="centos"
+// ID=ubuntu
+bool get_linux_distro_name(std::string& distro_name) {
+    std::map<std::string, std::string> parsed_file;
+
+    if (!parse_os_release_into_map(parsed_file)) {
+        return false;
+    }
+
+    auto itr = parsed_file.find("ID");
+
+    if (itr == parsed_file.end()) {
+        return false;
+    }
+
+    distro_name = itr->second;
+    return true;
+}
+
+
+
+// Returns Linux distro version
+// VERSION_ID="11"
+// VERSION_ID="8"
+// VERSION_ID="7"
+// VERSION_ID="16.04"
+bool get_linux_distro_version(std::string& distro_version) {
+    std::map<std::string, std::string> parsed_file;
+
+    if (!parse_os_release_into_map(parsed_file)) {
+        return false;
+    }
+
+    auto itr = parsed_file.find("VERSION_ID");
+
+    if (itr == parsed_file.end()) {
+        return false;
+    }
+
+    distro_version = itr->second;
+    return true;
+}
+
+
+// We will store option name as key and value will be value
+bool parse_os_release_into_map(std::map<std::string, std::string>& parsed_os_release) {
+    extern log4cpp::Category& logger;
+
+    // Format: https://www.freedesktop.org/software/systemd/man/os-release.html
+    std::ifstream os_release_file("/etc/os-release");
+
+    // Split line like:
+    // ID="centos"
+    boost::regex os_release_pattern("^(.*?)=\"?(.*?)\"?$", boost::regex::icase);
+
+    if (!os_release_file.is_open()) {
+        logger << log4cpp::Priority::ERROR << "Could not open /etc/os-release file";
+        return false;
+    }
+
+    std::string line;
+
+    while (getline(os_release_file, line)) {
+        // ID="centos"
+        // VERSION_ID="7"
+
+        boost::match_results<std::string::const_iterator> regex_results;
+
+        if (boost::regex_match(line, regex_results, os_release_pattern)) {
+            std::string value = regex_results[2];
+
+            // We may have or may not have quotes for value, strip them
+            boost::replace_all(value, "\"", "");
+
+            parsed_os_release[regex_results[1]] = value;
+        }
+    }
+
+    return true;
+}
+
+// Returns virtualisation method or "unknown"
+// It may have dash in value like "vm-other" or "lxc-libvirt" but no other symbols are expected
+std::string get_virtualisation_method() {
+    std::string error_text;
+    std::vector<std::string> output;
+
+    bool exec_result = exec("systemd-detect-virt --vm", output, error_text);
+
+    if (!exec_result) {
+        return "unknown";
+    }
+
+    if (output.empty()) {
+        return "unknown";
+    }
+
+    // Return first element
+    return boost::algorithm::to_lower_copy(output[0]);
+}
+
+// Get linux kernel version in form: 3.19.0-25-generic
+bool get_kernel_version(std::string& kernel_version) {
+    struct utsname current_utsname;
+
+    int uname_result = uname(&current_utsname);
+
+    if (uname_result != 0) {
+        return false;
+    }
+
+    // Release field is a char array (char release[], http://man7.org/linux/man-pages/man2/uname.2.html) and we do not need NULL check here
+    kernel_version = std::string(current_utsname.release);
+
+    return true;
+}
+
+// Returns all CPU flags in vector
+bool get_cpu_flags(std::vector<std::string>& flags) {
+    extern log4cpp::Category& logger;
+
+    std::ifstream cpuinfo_file("/proc/cpuinfo");
+    boost::regex processor_flags_pattern("^flags\\s+:\\s(.*?)$");
+
+    if (!cpuinfo_file.is_open()) {
+        logger << log4cpp::Priority::ERROR << "Could not open cpuinfo";
+        return false;
+    }
+
+    std::string line;
+    while (getline(cpuinfo_file, line)) {
+        boost::match_results<std::string::const_iterator> regex_results;
+
+        if (boost::regex_match(line, regex_results, processor_flags_pattern)) {
+            // Split all flags by space
+            split(flags, regex_results[1], boost::is_any_of(" "), boost::token_compress_on);
+            return true;
+        }
+    }
+
+    logger << log4cpp::Priority::ERROR << "Cannot find any flags in cpuinfo";
+    return false;
+}
+
+std::string get_cpu_model() {
+    extern log4cpp::Category& logger;
+
+    std::ifstream cpuinfo_file("/proc/cpuinfo");
+    boost::regex processor_model_pattern("^model name\\s+:\\s(.*?)$");
+
+    if (!cpuinfo_file.is_open()) {
+        logger << log4cpp::Priority::ERROR << "License: could not open cpuinfo";
+        return "";
+    }
+
+    std::string line;
+    while (getline(cpuinfo_file, line)) {
+        boost::match_results<std::string::const_iterator> regex_results;
+
+        if (boost::regex_match(line, regex_results, processor_model_pattern)) {
+            return regex_results[1];
+        }
+    }
+
+    // For new ARMs (Cavium Thunder X for example) we do not have model name in 4.10 kernel
+#ifdef __aarch64__
+    std::string implementer;
+    std::string part;
+    std::string revision;
+
+    boost::regex implementer_pattern("^CPU implementer\\s+:\\s(.*?)$");
+    boost::regex part_pattern("^CPU part\\s+:\\s(.*?)$");
+    boost::regex revision_pattern("^CPU revision\\s+:\\s(.*?)$");
+
+    // Reset to start of file
+    cpuinfo_file.clear();
+    cpuinfo_file.seekg(0, std::ios::beg);
+
+    while (getline(cpuinfo_file, line)) {
+        boost::match_results<std::string::const_iterator> regex_results_implementer;
+        boost::match_results<std::string::const_iterator> regex_results_part;
+        boost::match_results<std::string::const_iterator> regex_results_revision;
+
+        if (boost::regex_match(line, regex_results_implementer, implementer_pattern)) {
+            implementer = regex_results_implementer[1];
+        }
+
+        if (boost::regex_match(line, regex_results_part, part_pattern)) {
+            part = regex_results_part[1];
+        }
+
+        if (boost::regex_match(line, regex_results_revision, revision_pattern)) {
+            revision = regex_results_revision[1];
+        }
+    }
+
+    // If we fould all of them, use these fields as model
+    if (implementer.size() > 0 && part.size() > 0 && revision.size() > 0) {
+        return "implementer: " + implementer + " part: " + part + " revision: " + revision;
+    } else {
+        // logger << log4cpp::Priority::ERROR << "implementer: " << implementer << " part: " << part << " revision: " << revision;
+    }
+#endif
+
+    return "";
+}
+
