@@ -24,14 +24,13 @@
 #include <sys/stat.h>
 
 #include "bgp_protocol.hpp"
-#include "fast_library.h"
-#include "fastnetmon_packet_parser.h"
-#include "fastnetmon_types.h"
+#include "fast_library.hpp"
+#include "fastnetmon_types.hpp"
 #include "libpatricia/patricia.hpp"
-#include "packet_storage.h"
+#include "packet_storage.hpp"
 
 // Here we store variables which differs for different paltforms
-#include "fast_platform.h"
+#include "fast_platform.hpp"
 
 #include "fastnetmon_logic.hpp"
 
@@ -54,24 +53,28 @@
 #endif
 
 // Plugins
-#include "netflow_plugin/netflow_collector.h"
-#include "pcap_plugin/pcap_collector.h"
-#include "sflow_plugin/sflow_collector.h"
+#include "netflow_plugin/netflow_collector.hpp"
+#include "pcap_plugin/pcap_collector.hpp"
+#include "sflow_plugin/sflow_collector.hpp"
 
 #ifdef NETMAP_PLUGIN
-#include "netmap_plugin/netmap_collector.h"
+#include "netmap_plugin/netmap_collector.hpp"
+#endif
+
+#ifdef FASTNETMON_ENABLE_AF_XDP
+#include "xdp_plugin/xdp_collector.hpp"
 #endif
 
 #ifdef FASTNETMON_ENABLE_AFPACKET
-#include "afpacket_plugin/afpacket_collector.h"
+#include "afpacket_plugin/afpacket_collector.hpp"
 #endif
 
 #ifdef ENABLE_GOBGP
-#include "actions/gobgp_action.h"
+#include "actions/gobgp_action.hpp"
 #endif
 
 // Yes, maybe it's not an good idea but with this we can guarantee working code in example plugin
-#include "example_plugin/example_collector.h"
+#include "example_plugin/example_collector.hpp"
 
 #include <algorithm>
 #include <fstream>
@@ -81,6 +84,7 @@
 #include <sstream>
 #include <utility>
 #include <vector>
+#include <filesystem>
 
 #include <boost/regex.hpp>
 #include <boost/thread.hpp>
@@ -88,12 +92,14 @@
 
 #include <boost/program_options.hpp>
 
-#include "all_logcpp_libraries.h"
+#include "all_logcpp_libraries.hpp"
 
 // Boost libs
 #include <boost/algorithm/string.hpp>
 #include <boost/algorithm/string/trim.hpp>
 
+#include <boost/stacktrace.hpp>
+
 #ifdef GEOIP
 #include "GeoIP.h"
 #endif
@@ -102,7 +108,7 @@
 #include <hiredis/hiredis.h>
 #endif
 
-#include "packet_bucket.h"
+#include "packet_bucket.hpp"
 
 #include "ban_list.hpp"
 
@@ -128,9 +134,17 @@ std::string cli_stats_file_path = "/tmp/fastnetmon.dat";
 
 std::string cli_stats_ipv6_file_path = "/tmp/fastnetmon_ipv6.dat";
 
+// How often we send usage data 
 unsigned int stats_thread_sleep_time         = 3600;
+
+// Delay before we send first report about usage
 unsigned int stats_thread_initial_call_delay = 30;
 
+std::string reporting_server = "community-stats.fastnetmon.com";
+
+// Path to temporarily store backtrace when fatal failure happened
+std::string backtrace_path = "/var/log/fastnetmon_backtrace.dump";
+
 // Each this seconds we will check about available data in bucket
 unsigned int check_for_availible_for_processing_packets_buckets = 1;
 
@@ -148,9 +162,6 @@ unsigned int maximum_time_since_bucket_start_to_remove = 120;
 
 FastnetmonPlatformConfigurtion fastnetmon_platform_configuration;
 
-// Send or not any details about attack for ban script call over stdin
-bool notify_script_pass_details = true;
-
 bool notify_script_enabled = true;
 
 // We could collect attack dumps in pcap format
@@ -198,18 +209,12 @@ bool monitor_local_ip_addresses = true;
 // Enable monitoring for OpenVZ VPS IP addresses by reading their list from kernel
 bool monitor_openvz_vps_ip_addresses = false;
 
-// Trigger for enable or disable traffic counting for whole subnets
-bool enable_subnet_counters = false;
-
 // We will announce whole subnet instead single IP with BGP if this flag enabled
 bool exabgp_announce_whole_subnet = false;
 
 // We will announce only /32 host
 bool exabgp_announce_host = false;
 
-// With this flag we will announce more specfic then whole block Flow Spec announces
-bool exabgp_flow_spec_announces = false;
-
 ban_settings_t global_ban_settings;
 
 void init_global_ban_settings() {
@@ -244,6 +249,7 @@ void init_global_ban_settings() {
 bool enable_connection_tracking = true;
 
 bool enable_afpacket_collection         = false;
+bool enable_af_xdp_collection = false;
 bool enable_data_collection_from_mirror = true;
 bool enable_netmap_collection           = false;
 bool enable_sflow_collection            = false;
@@ -294,9 +300,6 @@ int global_ban_time = 1800;
 // We calc average pps/bps for this time
 double average_calculation_amount = 15;
 
-// We calc average pps/bps for subnets with this time, we use longer value for calculation average network traffic
-double average_calculation_amount_for_subnets = 30;
-
 // Key used for sorting clients in output.  Allowed sort params: packets/bytes/flows
 std::string sort_parameter = "bytes";
 
@@ -312,21 +315,11 @@ unsigned int number_of_packets_for_pcap_attack_dump = 500;
 // log file
 log4cpp::Category& logger = log4cpp::Category::getRoot();
 
-// We store all active BGP Flow Spec announces here
-active_flow_spec_announces_t active_flow_spec_announces;
-
 /* Configuration block ends */
 
 // We count total number of incoming/outgoing/internal and other traffic type packets/bytes
-// And initilize by 0 all fields
-total_counter_element_t total_counters[4];
-total_counter_element_t total_speed_counters[4];
-total_counter_element_t total_speed_average_counters[4];
-
-// IPv6 versions of total counters
-total_counter_element_t total_counters_ipv6[4];
-total_counter_element_t total_speed_counters_ipv6[4];
-total_counter_element_t total_speed_average_counters_ipv6[4];
+total_speed_counters_t total_counters_ipv4;
+total_speed_counters_t total_counters_ipv6;
 
 // Total amount of non parsed packets
 uint64_t total_unparsed_packets       = 0;
@@ -358,21 +351,21 @@ abstract_subnet_counters_t<subnet_ipv6_cidr_mask_t> ipv6_subnet_counters;
 // Host counters for IPv6
 abstract_subnet_counters_t<subnet_ipv6_cidr_mask_t> ipv6_host_counters;
 
-// Here we store taffic per subnet
-map_for_subnet_counters_t PerSubnetCountersMap;
-
-// Here we store traffic speed per subnet
-map_for_subnet_counters_t PerSubnetSpeedMap;
-
-// Here we store average speed per subnet
-map_for_subnet_counters_t PerSubnetAverageSpeedMap;
+// Here we store traffic per subnet
+abstract_subnet_counters_t<subnet_cidr_mask_t> ipv4_network_counters;
 
 // Flow tracking structures
 map_of_vector_counters_for_flow_t SubnetVectorMapFlow;
 
+std::string netflow_ipfix_all_protocols_total_flows_speed_desc = "Number of IPFIX and Netflow per second";
+int64_t netflow_ipfix_all_protocols_total_flows_speed          = 0;
+
+std::string sflow_raw_packet_headers_total_speed_desc = "Number of sFlow headers per second";
+int64_t sflow_raw_packet_headers_total_speed          = 0;
+
 /* End of our data structs */
-boost::mutex ban_list_details_mutex;
-boost::mutex ban_list_mutex;
+std::mutex ban_list_details_mutex;
+std::mutex ban_list_mutex;
 std::mutex flow_counter;
 
 // map for flows
@@ -451,7 +444,6 @@ bool process_outgoing_traffic = true;
 
 logging_configuration_t read_logging_settings(configuration_map_t configuration_map);
 std::string get_amplification_attack_type(amplification_attack_type_t attack_type);
-std::string generate_flow_spec_for_amplification_attack(amplification_attack_type_t amplification_attack_type, std::string destination_ip);
 ban_settings_t read_ban_settings(configuration_map_t configuration_map, std::string host_group_name = "");
 bool load_configuration_file();
 void free_up_all_resources();
@@ -494,8 +486,7 @@ void RunApiServer() {
 
 void sigpipe_handler_for_popen(int signo) {
     logger << log4cpp::Priority::ERROR << "Sorry but we experienced error with popen. "
-           << "Please check your scripts. They should receive data on stdin! Optionally you could disable "
-              "passing any details with configuration param: notify_script_pass_details = no";
+           << "Please check your scripts. They must receive data on stdin";
 
     // Well, we do not need exit here because we have another options to notifying about atatck
     // exit(1);
@@ -663,11 +654,6 @@ bool load_configuration_file() {
         average_calculation_amount = convert_string_to_integer(configuration_map["average_calculation_time"]);
     }
 
-    if (configuration_map.count("average_calculation_time_for_subnets") != 0) {
-        average_calculation_amount_for_subnets =
-            convert_string_to_integer(configuration_map["average_calculation_time_for_subnets"]);
-    }
-
     if (configuration_map.count("speed_calculation_delay") != 0) {
         recalculate_speed_timeout = convert_string_to_integer(configuration_map["speed_calculation_delay"]);
     }
@@ -756,10 +742,6 @@ bool load_configuration_file() {
             exabgp_enabled = false;
         }
 
-        if (configuration_map.count("exabgp_flow_spec_announces") != 0) {
-            exabgp_flow_spec_announces = configuration_map["exabgp_flow_spec_announces"] == "on";
-        }
-
         if (exabgp_enabled) {
             logger << log4cpp::Priority::INFO << "ExaBGP support initialized correctly";
         }
@@ -789,10 +771,6 @@ bool load_configuration_file() {
         exabgp_announce_host = configuration_map["exabgp_announce_host"] == "on" ? true : false;
     }
 
-    if (configuration_map.count("enable_subnet_counters") != 0) {
-        enable_subnet_counters = configuration_map["enable_subnet_counters"] == "on" ? true : false;
-    }
-
     // Graphite
     if (configuration_map.count("graphite") != 0) {
         graphite_enabled = configuration_map["graphite"] == "on" ? true : false;
@@ -859,6 +837,14 @@ bool load_configuration_file() {
         }
     }
 
+    if (configuration_map.count("mirror_afxdp") != 0) { 
+        if (configuration_map["mirror_afxdp"] == "on") {
+            enable_af_xdp_collection = true;
+        } else {
+            enable_af_xdp_collection = false;
+        }    
+    } 
+
     if (configuration_map.count("mirror_netmap") != 0) {
         if (configuration_map["mirror_netmap"] == "on") {
             enable_netmap_collection = true;
@@ -871,6 +857,11 @@ bool load_configuration_file() {
         enable_afpacket_collection = configuration_map["mirror_afpacket"] == "on";
     }
 
+    if (enable_afpacket_collection && enable_af_xdp_collection) {
+        logger << log4cpp::Priority::ERROR << "You cannot use AF_XDP and AF_PACKET in same time, select one";
+        exit(1);
+    }
+
     if (enable_netmap_collection && enable_data_collection_from_mirror) {
         logger << log4cpp::Priority::ERROR
                << "You have enabled pfring and netmap data collection "
@@ -977,10 +968,6 @@ bool load_configuration_file() {
         fastnetmon_platform_configuration.notify_script_path = configuration_map["notify_script_path"];
     }
 
-    if (configuration_map.count("notify_script_pass_details") != 0) {
-        notify_script_pass_details = configuration_map["notify_script_pass_details"] == "on" ? true : false;
-    }
-
     if (file_exists(fastnetmon_platform_configuration.notify_script_path)) {
         notify_script_enabled = true;
     } else {
@@ -1024,7 +1011,7 @@ void subnet_vectors_allocator(prefix_t* prefix, void* data) {
 
     subnet_cidr_mask_t current_subnet(subnet_as_integer, bitlen);
 
-    map_element_t zero_map_element{};
+    subnet_counter_t zero_map_element{};
 
     // Initilize our counters with fill constructor
     try {
@@ -1039,16 +1026,13 @@ void subnet_vectors_allocator(prefix_t* prefix, void* data) {
     // Initilize map element
     SubnetVectorMapFlow[current_subnet] = vector_of_flow_counters_t(network_size_in_ips);
 
-    // On creating it initilizes by zeros
+    // On creating it initializes by zeros
     conntrack_main_struct_t zero_conntrack_main_struct;
     std::fill(SubnetVectorMapFlow[current_subnet].begin(), SubnetVectorMapFlow[current_subnet].end(), zero_conntrack_main_struct);
-
-    PerSubnetCountersMap[current_subnet] = zero_map_element;
-    PerSubnetSpeedMap[current_subnet]    = zero_map_element;
 }
 
 void zeroify_all_counters() {
-    map_element_t zero_map_element{};
+    subnet_counter_t zero_map_element{};
 
     for (map_of_vector_counters_t::iterator itr = SubnetVectorMap.begin(); itr != SubnetVectorMap.end(); ++itr) {
         // logger<< log4cpp::Priority::INFO<<"Zeroify "<<itr->first;
@@ -1074,7 +1058,7 @@ bool load_our_networks_list() {
                 continue;
             }
             network_entries++;
-            make_and_lookup(whitelist_tree_ipv4, const_cast<char*>(text_subnet.c_str()));
+            make_and_lookup(whitelist_tree_ipv4, text_subnet.c_str());
         }
 
         logger << log4cpp::Priority::INFO << "We loaded " << network_entries << " networks from whitelist file";
@@ -1197,7 +1181,7 @@ bool load_our_networks_list() {
             network_address_in_cidr_form = new_network_address_as_string;
         }
 
-        make_and_lookup(lookup_tree_ipv4, const_cast<char*>(network_address_in_cidr_form.c_str()));
+        make_and_lookup(lookup_tree_ipv4, network_address_in_cidr_form.c_str());
     }
 
     for (std::vector<std::string>::iterator ii = networks_list_ipv6_as_string.begin();
@@ -1211,7 +1195,7 @@ bool load_our_networks_list() {
            << "Total number of monitored hosts (total size of all networks): " << total_number_of_hosts_in_our_networks;
 
     // 3 - speed counter, average speed counter and data counter
-    uint64_t memory_requirements = 3 * sizeof(map_element_t) * total_number_of_hosts_in_our_networks / 1024 / 1024;
+    uint64_t memory_requirements = 3 * sizeof(subnet_counter_t) * total_number_of_hosts_in_our_networks / 1024 / 1024;
 
     logger << log4cpp::Priority::INFO << "We need " << memory_requirements << " MB of memory for storing counters for your networks";
 
@@ -1346,6 +1330,24 @@ void init_logging(bool log_to_console) {
     logger << log4cpp::Priority::INFO << "Logger initialized!";
 }
 
+void reconfigure_logging_level(const std::string& logging_level) {
+    // Configure logging level
+    log4cpp::Priority::Value priority = log4cpp::Priority::INFO;
+
+    if (logging_level == "debug") {
+        priority = log4cpp::Priority::DEBUG;
+        logger << log4cpp::Priority::DEBUG << "Setting logging level to debug";
+    } else if (logging_level == "info" || logging_level == "") {
+        // It may be set to empty value in old versions before we introduced this flag
+        logger << log4cpp::Priority::DEBUG << "Setting logging level to info";
+        priority = log4cpp::Priority::INFO;
+    } else {
+        logger << log4cpp::Priority::ERROR << "Unknown logging level: " << logging_level;
+    }
+
+    logger.setPriority(priority);
+}
+
 void reconfigure_logging() {
     log4cpp::PatternLayout* layout = new log4cpp::PatternLayout();
     layout->setConversionPattern("[%p] %m%n");
@@ -1366,8 +1368,10 @@ void reconfigure_logging() {
         remote_syslog_appender->setLayout(layout);
         logger.addAppender(remote_syslog_appender);
 
-        logger << log4cpp::Priority::INFO << "We start remote syslog logging corectly";
+        logger << log4cpp::Priority::INFO << "We start remote syslog logging correctly";
     }
+
+    reconfigure_logging_level(logging_configuration.logging_level);
 }
 
 // Call fork function
@@ -1414,6 +1418,13 @@ void redirect_fds() {
 #pragma GCC diagnostic pop
 }
 
+// Handles fatal failure of FastNetMon's daemon
+void fatal_signal_handler(int signum) {
+    ::signal(signum, SIG_DFL);
+    boost::stacktrace::safe_dump_to(backtrace_path.c_str());
+    ::raise(SIGABRT);
+}
+
 int main(int argc, char** argv) {
     bool daemonize                = false;
     bool only_configuration_check = false;
@@ -1520,8 +1531,25 @@ int main(int argc, char** argv) {
     // enable core dumps
     enable_core_dumps();
 
+    // Setup fatal signal handlers to gracefully capture them
+    ::signal(SIGSEGV, &fatal_signal_handler);
+    ::signal(SIGABRT, &fatal_signal_handler);
+
     init_logging(log_to_console);
 
+    if (std::filesystem::exists(backtrace_path)) {
+        // there is a backtrace
+        std::ifstream ifs(backtrace_path);
+
+        boost::stacktrace::stacktrace st = boost::stacktrace::stacktrace::from_dump(ifs);
+        logger << log4cpp::Priority::ERROR << "Previous run crashed, you can find stack trace below";
+        logger << log4cpp::Priority::ERROR << st;
+
+        // cleaning up
+        ifs.close();
+        std::filesystem::remove(backtrace_path);
+    }
+
 #ifdef FASTNETMON_API
     gpr_set_log_function(silent_logging_function);
 #endif
@@ -1587,18 +1615,6 @@ int main(int argc, char** argv) {
     lookup_tree_ipv6    = New_Patricia(128);
     whitelist_tree_ipv6 = New_Patricia(128);
 
-    // nullify total counters
-    for (int index = 0; index < 4; index++) {
-        total_counters[index].bytes   = 0;
-        total_counters[index].packets = 0;
-
-        total_speed_counters[index].bytes   = 0;
-        total_speed_counters[index].packets = 0;
-
-        total_speed_average_counters[index].bytes   = 0;
-        total_speed_average_counters[index].packets = 0;
-    }
-
     /* Create folder for attack details */
     if (!folder_exists(fastnetmon_platform_configuration.attack_details_folder)) {
         logger << log4cpp::Priority::ERROR
@@ -1667,8 +1683,28 @@ int main(int argc, char** argv) {
     // Set inaccurate time value which will be used in process_packet() from capture backends
     time(&current_inaccurate_time);
 
-    // Start system speed recalculation thread
-    service_thread_group.add_thread(new boost::thread(system_counters_speed_thread_handler));
+    // start thread which pre-calculates speed for system counters
+    auto system_counters_speed_thread = new boost::thread(system_counters_speed_thread_handler);
+    set_boost_process_name(system_counters_speed_thread, "metrics_speed");
+    service_thread_group.add_thread(system_counters_speed_thread);
+
+
+    auto inaccurate_time_generator_thread = new boost::thread(inaccurate_time_generator);
+    set_boost_process_name(inaccurate_time_generator_thread, "fast_time");
+    service_thread_group.add_thread(inaccurate_time_generator_thread);
+
+    // Run stats thread
+    bool usage_stats = true;
+
+    if (configuration_map.count("disable_usage_report") != 0 && configuration_map["disable_usage_report"] == "on") {
+	    usage_stats = false;
+    }
+
+    if (usage_stats) {
+        auto stats_thread = new boost::thread(collect_stats);
+        set_boost_process_name(stats_thread, "stats");
+        service_thread_group.add_thread(stats_thread);
+    }
 
     // Run screen draw thread for IPv4
     service_thread_group.add_thread(new boost::thread(screen_draw_ipv4_thread));
@@ -1712,6 +1748,14 @@ int main(int argc, char** argv) {
     }
 #endif
 
+#ifdef FASTNETMON_ENABLE_AF_XDP
+    if (enable_af_xdp_collection) {
+        auto xdp_thread = new boost::thread(start_xdp_collection, process_packet);
+        set_boost_process_name(xdp_thread, "xdp");
+        packet_capture_plugin_thread_group.add_thread(xdp_thread);
+    }
+#endif
+
     if (enable_sflow_collection) {
         packet_capture_plugin_thread_group.add_thread(new boost::thread(start_sflow_collection, process_packet));
     }
