@@ -1,23 +1,23 @@
 #include "influxdb.hpp"
 
 #include "../abstract_subnet_counters.hpp"
-#include "../fast_library.h"
-#include "../fastnetmon_types.h"
+#include "../fast_library.hpp"
+#include "../fastnetmon_types.hpp"
 
-#include "../all_logcpp_libraries.h"
+#include "../all_logcpp_libraries.hpp"
+
+#include "../abstract_subnet_counters.hpp"
 
 #include <vector>
 
 extern struct timeval graphite_thread_execution_time;
-extern total_counter_element_t total_speed_average_counters[4];
 extern map_of_vector_counters_t SubnetVectorMapSpeed;
 extern map_of_vector_counters_t SubnetVectorMapSpeedAverage;
 extern uint64_t incoming_total_flows_speed;
 extern uint64_t outgoing_total_flows_speed;
-extern map_for_subnet_counters_t PerSubnetAverageSpeedMap;
+extern abstract_subnet_counters_t<subnet_cidr_mask_t> ipv4_network_counters;
 extern uint64_t influxdb_writes_total;
 extern uint64_t influxdb_writes_failed;
-extern total_counter_element_t total_speed_average_counters_ipv6[4];
 extern abstract_subnet_counters_t<subnet_ipv6_cidr_mask_t> ipv6_host_counters;
 extern abstract_subnet_counters_t<subnet_cidr_mask_t> ipv4_host_counters;
 extern abstract_subnet_counters_t<subnet_cidr_mask_t> ipv4_remote_host_counters;
@@ -25,6 +25,9 @@ extern std::vector<ban_settings_t> hostgroup_list_total_calculation;
 extern std::mutex hostgroup_list_total_calculation_mutex;
 extern abstract_subnet_counters_t<int64_t> per_hostgroup_total_counters;
 extern log4cpp::Category& logger;
+extern total_speed_counters_t total_counters_ipv4;
+extern total_speed_counters_t total_counters_ipv6;
+
 
 extern std::string influxdb_database;
 extern std::string influxdb_host;
@@ -170,7 +173,7 @@ void influxdb_push_thread() {
         // First of all push total counters to InfluxDB
         push_total_traffic_counters_to_influxdb(influxdb_database, current_influxdb_ip_address,
                                                 std::to_string(influxdb_port), influxdb_auth, influxdb_user,
-                                                influxdb_password, "total_traffic", total_speed_average_counters, false);
+                                                influxdb_password, "total_traffic", total_counters_ipv4.total_speed_average_counters, false);
 
         // Push per subnet counters to InfluxDB
         push_network_traffic_counters_to_influxdb(influxdb_database, current_influxdb_ip_address, std::to_string(influxdb_port),
@@ -190,7 +193,7 @@ void influxdb_push_thread() {
         // Push total IPv6 counters
         push_total_traffic_counters_to_influxdb(influxdb_database, current_influxdb_ip_address,
                                                 std::to_string(influxdb_port), influxdb_auth, influxdb_user, influxdb_password,
-                                                "total_traffic_ipv6", total_speed_average_counters_ipv6, true);
+                                                "total_traffic_ipv6", total_counters_ipv6.total_speed_average_counters, true);
     }
 }
 
@@ -274,7 +277,7 @@ bool push_hosts_traffic_counters_to_influxdb(std::string influx_database,
             std::string client_ip_as_string = convert_ip_as_uint_to_string(client_ip);
 
             // Here we could have average or instantaneous speed
-            map_element_t* current_speed_element = &*vector_itr;
+            subnet_counter_t* current_speed_element = &*vector_itr;
 
             // Skip elements with zero speed
             if (current_speed_element->is_zero()) {
@@ -342,11 +345,15 @@ bool push_network_traffic_counters_to_influxdb(std::string influx_database,
                                                bool enable_auth,
                                                std::string influx_user,
                                                std::string influx_password) {
-    for (map_for_subnet_counters_t::iterator itr = PerSubnetAverageSpeedMap.begin(); itr != PerSubnetAverageSpeedMap.end(); ++itr) {
+
+    std::vector<std::pair<subnet_cidr_mask_t, subnet_counter_t>> speed_elements;
+    ipv4_network_counters.get_all_non_zero_average_speed_elements_as_pairs(speed_elements);
+
+    for (const auto& itr: speed_elements) {
         std::map<std::string, uint64_t> plain_total_counters_map;
 
-        map_element_t* speed         = &itr->second;
-        std::string subnet_as_string = convert_subnet_to_string(itr->first);
+        const subnet_counter_t* speed         = &itr.second;
+        std::string subnet_as_string = convert_subnet_to_string(itr.first);
 
         fill_main_counters_for_influxdb(speed, plain_total_counters_map, false);
 
@@ -390,46 +397,46 @@ bool write_line_of_data_to_influxdb(std::string influx_database,
 }
 
 // Fills special structure which we use to export metrics into InfluxDB
-void fill_per_protocol_countres_for_influxdb(const map_element_t* current_speed_element,
+void fill_per_protocol_countres_for_influxdb(const subnet_counter_t* current_speed_element,
                                              std::map<std::string, uint64_t>& plain_total_counters_map) {
-    plain_total_counters_map["fragmented_packets_incoming"] = current_speed_element->fragmented_in_packets;
-    plain_total_counters_map["tcp_packets_incoming"]        = current_speed_element->tcp_in_packets;
-    plain_total_counters_map["tcp_syn_packets_incoming"]    = current_speed_element->tcp_syn_in_packets;
-    plain_total_counters_map["udp_packets_incoming"]        = current_speed_element->udp_in_packets;
-    plain_total_counters_map["icmp_packets_incoming"]       = current_speed_element->icmp_in_packets;
+    plain_total_counters_map["fragmented_packets_incoming"] = current_speed_element->fragmented.in_packets;
+    plain_total_counters_map["tcp_packets_incoming"]        = current_speed_element->tcp.in_packets;
+    plain_total_counters_map["tcp_syn_packets_incoming"]    = current_speed_element->tcp_syn.in_packets;
+    plain_total_counters_map["udp_packets_incoming"]        = current_speed_element->udp.in_packets;
+    plain_total_counters_map["icmp_packets_incoming"]       = current_speed_element->icmp.in_packets;
 
-    plain_total_counters_map["fragmented_bits_incoming"] = current_speed_element->fragmented_in_bytes * 8;
-    plain_total_counters_map["tcp_bits_incoming"]        = current_speed_element->tcp_in_bytes * 8;
-    plain_total_counters_map["tcp_syn_bits_incoming"]    = current_speed_element->tcp_syn_in_bytes * 8;
-    plain_total_counters_map["udp_bits_incoming"]        = current_speed_element->udp_in_bytes * 8;
-    plain_total_counters_map["icmp_bits_incoming"]       = current_speed_element->icmp_in_bytes * 8;
+    plain_total_counters_map["fragmented_bits_incoming"] = current_speed_element->fragmented.in_bytes * 8;
+    plain_total_counters_map["tcp_bits_incoming"]        = current_speed_element->tcp.in_bytes * 8;
+    plain_total_counters_map["tcp_syn_bits_incoming"]    = current_speed_element->tcp_syn.in_bytes * 8;
+    plain_total_counters_map["udp_bits_incoming"]        = current_speed_element->udp.in_bytes * 8;
+    plain_total_counters_map["icmp_bits_incoming"]       = current_speed_element->icmp.in_bytes * 8;
 
 
     // Outgoing
-    plain_total_counters_map["fragmented_packets_outgoing"] = current_speed_element->fragmented_out_packets;
-    plain_total_counters_map["tcp_packets_outgoing"]        = current_speed_element->tcp_out_packets;
-    plain_total_counters_map["tcp_syn_packets_outgoing"]    = current_speed_element->tcp_syn_out_packets;
-    plain_total_counters_map["udp_packets_outgoing"]        = current_speed_element->udp_out_packets;
-    plain_total_counters_map["icmp_packets_outgoing"]       = current_speed_element->icmp_out_packets;
+    plain_total_counters_map["fragmented_packets_outgoing"] = current_speed_element->fragmented.out_packets;
+    plain_total_counters_map["tcp_packets_outgoing"]        = current_speed_element->tcp.out_packets;
+    plain_total_counters_map["tcp_syn_packets_outgoing"]    = current_speed_element->tcp_syn.out_packets;
+    plain_total_counters_map["udp_packets_outgoing"]        = current_speed_element->udp.out_packets;
+    plain_total_counters_map["icmp_packets_outgoing"]       = current_speed_element->icmp.out_packets;
 
-    plain_total_counters_map["fragmented_bits_outgoing"] = current_speed_element->fragmented_out_bytes * 8;
-    plain_total_counters_map["tcp_bits_outgoing"]        = current_speed_element->tcp_out_bytes * 8;
-    plain_total_counters_map["tcp_syn_bits_outgoing"]    = current_speed_element->tcp_syn_out_bytes * 8;
-    plain_total_counters_map["udp_bits_outgoing"]        = current_speed_element->udp_out_bytes * 8;
-    plain_total_counters_map["icmp_bits_outgoing"]       = current_speed_element->icmp_out_bytes * 8;
+    plain_total_counters_map["fragmented_bits_outgoing"] = current_speed_element->fragmented.out_bytes * 8;
+    plain_total_counters_map["tcp_bits_outgoing"]        = current_speed_element->tcp.out_bytes * 8;
+    plain_total_counters_map["tcp_syn_bits_outgoing"]    = current_speed_element->tcp_syn.out_bytes * 8;
+    plain_total_counters_map["udp_bits_outgoing"]        = current_speed_element->udp.out_bytes * 8;
+    plain_total_counters_map["icmp_bits_outgoing"]       = current_speed_element->icmp.out_bytes * 8;
 }
 
 // Fills special structure which we use to export metrics into InfluxDB
-void fill_main_counters_for_influxdb(const map_element_t* current_speed_element,
+void fill_main_counters_for_influxdb(const subnet_counter_t* current_speed_element,
                                      std::map<std::string, uint64_t>& plain_total_counters_map,
                                      bool populate_flow) {
     // Prepare incoming traffic data
-    plain_total_counters_map["packets_incoming"] = current_speed_element->in_packets;
-    plain_total_counters_map["bits_incoming"]    = current_speed_element->in_bytes * 8;
+    plain_total_counters_map["packets_incoming"] = current_speed_element->total.in_packets;
+    plain_total_counters_map["bits_incoming"]    = current_speed_element->total.in_bytes * 8;
 
     // Outdoing traffic
-    plain_total_counters_map["packets_outgoing"] = current_speed_element->out_packets;
-    plain_total_counters_map["bits_outgoing"]    = current_speed_element->out_bytes * 8;
+    plain_total_counters_map["packets_outgoing"] = current_speed_element->total.out_packets;
+    plain_total_counters_map["bits_outgoing"]    = current_speed_element->total.out_bytes * 8;
 
     if (populate_flow) {
         plain_total_counters_map["flows_incoming"] = current_speed_element->in_flows;
