@@ -13,13 +13,13 @@
 #include <mutex>
 #include <vector>
 
-#include "../fast_library.h"
-#include "../ipfix_rfc.h"
+#include "../fast_library.hpp"
+#include "../ipfix_rfc.hpp"
 
-#include "../all_logcpp_libraries.h"
+#include "../all_logcpp_libraries.hpp"
 
 #include "netflow.hpp"
-#include "netflow_collector.h"
+#include "netflow_collector.hpp"
 
 #include <boost/serialization/map.hpp>
 #include <boost/serialization/vector.hpp>
@@ -28,7 +28,7 @@
 #include <boost/archive/xml_oarchive.hpp>
 
 // For Netflow lite parsing
-#include "../simple_packet_parser_ng.h"
+#include "../simple_packet_parser_ng.hpp"
 
 // Get it from main programme
 extern log4cpp::Category& logger;
@@ -67,8 +67,9 @@ uint64_t netflow_v9_total_ipv6_packets = 0;
 
 uint64_t netflow_ipfix_total_packets = 0;
 
-// Total number of flows summarized for all kinds of Netflow and IPFIX
-uint64_t netflow_all_protocols_total_flows = 0;
+std::string netflow_ipfix_all_protocols_total_flows_desc =
+    "Total number of flows summarized for all kinds of Netflow and IPFIX";
+uint64_t netflow_ipfix_all_protocols_total_flows = 0; 
 
 // Multiple in each UDP packet
 uint64_t netflow_ipfix_total_flows        = 0;
@@ -215,7 +216,7 @@ std::vector<system_counter_t> get_netflow_stats() {
     system_counter.push_back(system_counter_t("netflow_ipfix_total_ipv4_packets", netflow_ipfix_total_ipv4_packets));
     system_counter.push_back(system_counter_t("netflow_ipfix_total_ipv6_packets", netflow_ipfix_total_ipv6_packets));
 
-    system_counter.push_back(system_counter_t("netflow_all_protocols_total_flows", netflow_all_protocols_total_flows));
+    system_counter.push_back(system_counter_t("netflow_ipfix_all_protocols_total_flows", netflow_ipfix_all_protocols_total_flows));
 
     system_counter.push_back(system_counter_t("netflow_broken_packets", netflow_broken_packets));
     system_counter.push_back(system_counter_t("template_updates_number_due_to_real_changes", template_updates_number_due_to_real_changes));
@@ -987,8 +988,10 @@ int nf9_rec_to_flow(uint32_t record_type, uint32_t record_length, uint8_t* data,
             full_packet_length = flow_meta.data_link_frame_size;
         }
 
+        bool unpack_gre = false;
+
         auto result = parse_raw_packet_to_simple_packet_full_ng((u_char*)(data), full_packet_length, record_length,
-                                                                flow_meta.nested_packet, read_packet_length_from_ip_header);
+                                                                flow_meta.nested_packet, unpack_gre, read_packet_length_from_ip_header);
 
         if (result != network_data_stuctures::parser_code_t::success) {
             // Cannot decode data
@@ -1310,7 +1313,7 @@ void nf10_flowset_to_store(uint8_t* pkt,
         offset += record_length;
     }
 
-    netflow_all_protocols_total_flows++;
+    netflow_ipfix_all_protocols_total_flows++;
 
     netflow_ipfix_total_flows++;
 
@@ -1600,7 +1603,7 @@ void nf9_flowset_to_store(uint8_t* pkt,
     // Total number of Netflow v9 flows
     netflow_v9_total_flows++;
 
-    netflow_all_protocols_total_flows++;
+    netflow_ipfix_all_protocols_total_flows++;
 
     if (packet.ip_protocol_version == 4) {
         netflow_v9_total_ipv4_packets++;
@@ -2040,7 +2043,7 @@ bool process_netflow_packet_v5(uint8_t* packet, uint32_t len, std::string& clien
             return false;
         }
 
-        netflow_all_protocols_total_flows++;
+        netflow_ipfix_all_protocols_total_flows++;
         netflow_v5_total_flows++;
 
         /* Decode to host encoding */
