@@ -42,6 +42,9 @@ my $distro_architecture = '';
 
 my $gcc_version = '12.1.0';
 
+# We need it for all OpenSSL dependencies
+my $openssl_folder_name = "openssl_1_1_1q";
+
 my $user_email = '';
 
 my $install_log_path = "/tmp/fastnetmon_install_$$.log";
@@ -323,8 +326,6 @@ sub main {
     }
 
     if ($build_dependencies_only) {
-        install_json_c();
-
         install_openssl();
 
         install_capnproto();
@@ -340,6 +341,10 @@ sub main {
         install_boost_dependencies();
         install_boost();
 
+        install_libelf();
+
+        install_libbpf();
+
         if ($we_have_hiredis_support) {
            install_hiredis();
         }
@@ -463,40 +468,6 @@ sub download_file {
     }     
 }
 
-sub install_json_c {
-    my $archive_name  = 'json-c-0.13.1-20180305.tar.gz';
-    my $install_path = "$library_install_folder/json-c-0.13";
-
-    if (-e $install_path) {
-	    print "json-c was already installed\n";
-	    return 1;
-    }
-
-    print "Install json library\n";
-
-    chdir $temp_folder_for_building_project;
-
-    print "Download archive\n";
-
-    my $json_c_download_result = download_file("https://github.com/json-c/json-c/archive/$archive_name",
-        $archive_name,
-        'd4c667ab94e7051b1e78aa727f0d937caba971e9');
-
-    unless ($json_c_download_result) {
-        fast_die("Can't download json-c sources");
-    }
-
-    print "Uncompress it\n";
-    exec_command("tar -xf $archive_name");
-    chdir "json-c-json-c-0.13.1-20180305";
-
-    print "Build it\n";
-    exec_command("$configure_options ./configure --prefix=$install_path");
-
-    print "Install it\n";
-    exec_command("make $make_options install");
-}
-
 sub install_log4cpp {
     my $log_cpp_version_short = '1.1.3';
 
@@ -577,7 +548,7 @@ sub install_grpc {
 
     print "Build gRPC\n";
     # We need to specify PKG config path to pick up our custom OpenSSL instead of system one
-    my $make_result = exec_command("$ld_library_path_for_make PKG_CONFIG_PATH=$library_install_folder/openssl_1_0_2d/lib/pkgconfig make $make_options");
+    my $make_result = exec_command("$ld_library_path_for_make PKG_CONFIG_PATH=$library_install_folder/openssl_1_1_1q/lib/pkgconfig make $make_options");
 
     unless ($make_result) {
         fast_die( "Could not build gRPC: make failed\n");
@@ -759,12 +730,14 @@ sub install_mongo_client {
         return 1;
     }    
 
+    my $openssl_path = "$library_install_folder/$openssl_folder_name";
+
     # OpenSSL is mandatory for SCRAM-SHA-1 auth mode
     # I also use flag ENABLE_ICU=OFF to disable linking against icu system library. I do no think that we really need it
     my $res = install_cmake_based_software("https://github.com/mongodb/mongo-c-driver/releases/download/1.16.1/mongo-c-driver-1.16.1.tar.gz",
         "f9bd005195895538af821708112bf861090da354",
     $install_path,
-    "$ld_library_path_for_make $cmake_path -DENABLE_AUTOMATIC_INIT_AND_CLEANUP=OFF -DCMAKE_BUILD_TYPE=Release -DCMAKE_INSTALL_PREFIX:STRING=$library_install_folder/mongo_c_driver_1_16_1 -DCMAKE_C_COMPILER=$default_c_compiler_path -DOPENSSL_ROOT_DIR=$library_install_folder/openssl_1_0_2d -DCMAKE_CXX_COMPILER=$default_cpp_compiler_path -DENABLE_ICU=OFF ..", $ld_library_path_for_make);
+    "$ld_library_path_for_make $cmake_path -DENABLE_AUTOMATIC_INIT_AND_CLEANUP=OFF -DCMAKE_BUILD_TYPE=Release -DCMAKE_INSTALL_PREFIX:STRING=$library_install_folder/mongo_c_driver_1_16_1 -DCMAKE_C_COMPILER=$default_c_compiler_path -DOPENSSL_ROOT_DIR=$openssl_path -DCMAKE_CXX_COMPILER=$default_cpp_compiler_path -DENABLE_ICU=OFF ..", $ld_library_path_for_make);
 
     if (!$res) {
         die "Could not install mongo c client\n";
@@ -1134,8 +1107,8 @@ sub install_capnproto {
 }
 
 sub install_openssl {
-    my $distro_file_name = 'openssl-1.0.2d.tar.gz';
-    my $openssl_install_path = "$library_install_folder/openssl_1_0_2d";
+    my $distro_file_name = 'openssl-1.1.1q.tar.gz';
+    my $openssl_install_path = "$library_install_folder/openssl_1_1_1q";
  
     if (-e $openssl_install_path) {
         warn "We found already installed openssl in folder $openssl_install_path Skip compilation\n";
@@ -1144,15 +1117,15 @@ sub install_openssl {
 
     chdir $temp_folder_for_building_project;
    
-    my $openssl_download_result = download_file("https://www.openssl.org/source/old/1.0.2/$distro_file_name", 
-        $distro_file_name, 'd01d17b44663e8ffa6a33a5a30053779d9593c3d');
+    my $openssl_download_result = download_file("https://www.openssl.org/source/$distro_file_name", 
+        $distro_file_name, '79511a8f46f267c533efd32f22ad3bf89a92d8e5');
 
     unless ($openssl_download_result) {    
         die "Could not download openssl";
     }    
 
     exec_command("tar -xf $distro_file_name");
-    chdir "openssl-1.0.2d";
+    chdir "openssl-1.1.1q";
 
     exec_command("./config shared --prefix=$openssl_install_path");
     exec_command("make -j $make_options");
@@ -1167,11 +1140,13 @@ sub install_poco {
          return 1;
     }    
 
+    my $openssl_path = "$library_install_folder/$openssl_folder_name";
+
     # Actually it's not standard "configure". That is custom bash based script! And it's not handling options suitable for
     # configure (i.e. CC and other)
     my $res = install_configure_based_software("https://github.com/pocoproject/poco/archive/poco-1.10.0-release.tar.gz",
         'cc75c9ca9d21422683ee7d71c5a98aaf72b45bcc', "$library_install_folder/poco_1_10_0",
-        "--minimal --shared --no-samples --no-tests --include-path=$library_install_folder/openssl_1_0_2d/include --library-path=$library_install_folder/openssl_1_0_2d/lib --cflags=\"-std=c++11\"");
+        "--minimal --shared --no-samples --no-tests --include-path=$openssl_path/include --library-path=$openssl_path/lib --cflags=\"-std=c++11\"");
 
     unless ($res) {
         die "Could not install poco";
@@ -1273,9 +1248,9 @@ sub install_boost_builder {
     chdir $temp_folder_for_building_project;
 
     # We use another name because it uses same name as boost distribution
-    my $archive_file_name = '4.3.0.tar.gz';
+    my $archive_file_name = '4.9.2.tar.gz';
 
-    my $boost_builder_install_folder = "$library_install_folder/boost_build_4_3_0";
+    my $boost_builder_install_folder = "$library_install_folder/boost_build_4_9_2";
 
     if (-e $boost_builder_install_folder) {
         warn "Found installed Boost builder at $boost_builder_install_folder\n";
@@ -1283,8 +1258,8 @@ sub install_boost_builder {
     }
 
     print "Download boost builder\n";
-    my $boost_build_result = download_file("https://github.com/boostorg/build/archive/$archive_file_name", $archive_file_name,
-        '');
+    my $boost_build_result = download_file("https://github.com/bfgroup/build/archive/$archive_file_name", $archive_file_name,
+        '1c77d3fda9425fd89b783db8f7bd8ebecdf8f916');
 
     unless ($boost_build_result) {
         fast_die("Can't download boost builder\n");
@@ -1293,7 +1268,7 @@ sub install_boost_builder {
     print "Unpack boost builder\n";
     exec_command("tar -xf $archive_file_name");
 
-    unless (chdir "build-4.3.0") {
+    unless (chdir "b2-4.9.2") {
         fast_die("Cannot do chdir to build boost folder\n");
     }
 
@@ -1389,8 +1364,82 @@ sub install_boost_dependencies {
     }
 }
 
+sub install_libelf {
+    if (-e "$library_install_folder/elfutils_0_186") {
+        warn "elfutils already exists\n";
+        return 1;
+    }
+
+    if ($distro_type eq 'ubuntu' || $distro_type eq 'debian') {
+        apt_get(('zlib1g-dev'));
+    } elsif ($distro_type eq 'centos') {
+        yum('zlib-devel', 'm4');
+    }
+
+    my $res = install_configure_based_software("https://sourceware.org/pub/elfutils/0.186/elfutils-0.186.tar.bz2", "650d52024be684dabf18a5261a69836a16f84f72", "$library_install_folder/elfutils_0_186", '--disable-debuginfod --disable-libdebuginfod');
+
+    unless ($res) {
+        die "Cannot install elfutils\n";
+    }
+
+    return 1;
+}
+
+
+sub install_libbpf {
+    # TODO: use our own libbpf source for compilation
+    # Well, it's easy to say: https://github.com/libbpf/libbpf
+
+    if ($distro_type eq 'ubuntu' || $distro_type eq 'debian') {
+        my @dependency_list = ('libelf-dev');
+        apt_get(@dependency_list);
+    } elsif ($distro_type eq 'centos') {
+        yum('elfutils-libelf-devel');
+    }
+
+    # 5_16
+    my $libbpf_package_install_path = "$library_install_folder/libbpf_4_19";
+
+    if (-e $libbpf_package_install_path) {
+    warn "libbpf is installed, skip build\n";
+    return 1;
+    }
+
+    # linux-5.16.tar.xz
+    my $archive_file_name = 'linux-4.19.tar.xz';
+
+    print "Download Linux kernel\n";
+    chdir $temp_folder_for_building_project;
+
+    # //cdn.kernel.org/pub/linux/kernel/v5.x/linux-5.16.tar.xz
+    # fd1d61124af4f76980eeef7257f625c0366eb70a
+    my $linux_kernel_download_result = download_file("https://cdn.kernel.org/pub/linux/kernel/v4.x/linux-4.19.tar.xz",  $archive_file_name, '40f3e72192d59c0b7a4638cebd0ea55d35a782bb');
+
+    unless ($linux_kernel_download_result) {
+        die "Cannot download linux kernel\n";
+    }
+
+    print "Unpack kernel\n";
+    system("tar -xf $archive_file_name");
+
+    # 5.16
+    chdir "linux-4.19";
+
+    chdir "tools/lib/bpf";
+    system("make");
+
+    system("mkdir -p $libbpf_package_install_path");
+    system("cp libbpf.so libbpf.a $libbpf_package_install_path");
+
+    system("mkdir -p $libbpf_package_install_path/include/bpf");
+    system("cp bpf.h libbpf.h $libbpf_package_install_path/include/bpf");
+
+    return 1;
+}
+
+
 sub install_boost {
-    my $boost_install_path = "$library_install_folder/boost_1_78_0";
+    my $boost_install_path = "$library_install_folder/boost_1_80_0";
 
     if (-e $boost_install_path) {
         warn "Boost libraries already exist in $boost_install_path. Skip build process\n";
@@ -1398,14 +1447,14 @@ sub install_boost {
     }
 
     chdir $library_install_folder;
-    my $archive_file_name = 'boost_1_78_0.tar.gz';
+    my $archive_file_name = 'boost_1_80_0.tar.gz';
 
     print "Install Boost dependencies\n";
    
-    my $url_boost = "https://boostorg.jfrog.io/artifactory/main/release/1.78.0/source/boost_1_78_0.tar.bz2";
+    my $url_boost = "https://boostorg.jfrog.io/artifactory/main/release/1.80.0/source/boost_1_80_0.tar.bz2";
 
     print "Download Boost source code\n";
-    my $boost_download_result = download_file($url_boost, $archive_file_name, '7ccc47e82926be693810a687015ddc490b49296d');
+    my $boost_download_result = download_file($url_boost, $archive_file_name, '690a2a2ed6861129828984b1d52a473d2c8393d1');
 
     unless ($boost_download_result) {
         fast_die("Can't download Boost source code\n");
@@ -1414,7 +1463,7 @@ sub install_boost {
     print "Unpack Boost source code\n";
     exec_command("tar -xf $archive_file_name");
 
-    my $folder_name_inside_archive = 'boost_1_78_0';
+    my $folder_name_inside_archive = 'boost_1_80_0';
 
     print "Fix permissions\n";
     # Fix permissions because they are broken inside official archive
@@ -1437,7 +1486,7 @@ sub install_boost {
     print "Build Boost\n";
     # We have troubles when run this code with vzctl exec so we should add custom compiler in path 
     # linkflags is required to specify custom path to libicu from regexp library
-    my $b2_build_result = exec_command("$ld_library_path_for_make $library_install_folder/boost_build_4_3_0/bin/b2 -j $boost_build_threads -sICU_PATH=$library_install_folder/libicu_65_1 linkflags=\"-Wl,-rpath,$library_install_folder/libicu_65_1/lib\" --build-dir=$temp_folder_for_building_project/boost_build_temp_directory_1_7_8 link=shared --without-test --without-python --without-wave --without-log --without-mpi");
+    my $b2_build_result = exec_command("$ld_library_path_for_make $library_install_folder/boost_build_4_9_2/bin/b2 -j $boost_build_threads -sICU_PATH=$library_install_folder/libicu_65_1 linkflags=\"-Wl,-rpath,$library_install_folder/libicu_65_1/lib\" --build-dir=$temp_folder_for_building_project/boost_build_temp_directory_1_7_8 link=shared --without-test --without-python --without-wave --without-log --without-mpi");
 
     unless ($b2_build_result) {
         die "Can't execute b2 build correctly\n";
