@@ -5,8 +5,8 @@
 #include <sys/types.h>
 #include <type_traits>
 
-#include "../libsflow/libsflow.h"
-#include "sflow_collector.h"
+#include "../libsflow/libsflow.hpp"
+#include "sflow_collector.hpp"
 
 #include <arpa/inet.h>
 #include <netinet/in.h>
@@ -19,13 +19,13 @@
 #include <string.h>
 #include <sys/socket.h>
 
-#include "../fast_library.h"
+#include "../fast_library.hpp"
 
-#include "../all_logcpp_libraries.h"
+#include "../all_logcpp_libraries.hpp"
 
 extern log4cpp::Category& logger;
 
-#include "../simple_packet_parser_ng.h"
+#include "../simple_packet_parser_ng.hpp"
 
 // Global configuration map
 extern std::map<std::string, std::string> configuration_map;
@@ -354,10 +354,13 @@ bool process_sflow_flow_sample(uint8_t* data_pointer,
 
             if (sflow_raw_protocol_header.header_protocol == SFLOW_HEADER_PROTOCOL_ETHERNET) {
 
+                bool unpack_gre = false;
+
                 // We could enable this new parser for testing purpose
                 auto result = parse_raw_packet_to_simple_packet_full_ng(header_payload_pointer,
                                                                         sflow_raw_protocol_header.frame_length_before_sampling,
                                                                         sflow_raw_protocol_header.header_size, packet,
+                                                                        unpack_gre,
                                                                         sflow_read_packet_length_from_ip_header);
 
                 if (result != network_data_stuctures::parser_code_t::success) {
